/*
 * @Author: house 
 * @Date: 2019-07-27 14:26:07 
 * @Last Modified by: house
 * @Last Modified time: 2019-08-10 18:19:56
 * 可拖拽组件
 */




const {ccclass, property} = cc._decorator;


@ccclass
export default class DragComponent extends cc.Component {
    
    /**
     * 拖拽目标对象数组
     */
    @property({
        type:cc.Node,
        tooltip:'拖拽目标对象数组'
    })
    targetNodeList: cc.Node[] = [];

    // 拖拽结果
    @property({
        type:cc.Node,
        tooltip:'事件响应对象'
    })
    eventNode:cc.Node = null;

    // 拖拽成功回调
    dragSuccessCallback:Function;
    // 原始坐标
    private _pos:cc.Vec2;
    // 是否拖拽到目标对象
    private _isDragSuccess:boolean;
    // 150毫秒发送一次拖拽事件
    private _durTime = 150;
    private _timeStamp = new Date().getTime();
   
    private isMoved = false;
    private _zIndex = 0;
    
    // onLoad () {}
    start () {
        this._timeStamp = new Date().getTime();
        this._pos = this.node.position;
        this.node.on(cc.Node.EventType.TOUCH_START, this.onTouchStart, this);
        this.node.on(cc.Node.EventType.TOUCH_MOVE, this.onTouchMove, this);
        this.node.on(cc.Node.EventType.TOUCH_END, this.onTouchEnd, this);
        this.node.on(cc.Node.EventType.TOUCH_CANCEL, this.onTouchEnd, this);
        this._zIndex = this.node.zIndex;
       
        // this.node.on(DragEvent.Drag_Success, this.onDragSuccessHandler, this);
        
    }

    // 重置坐标
    refreshPos(){
        this.node.zIndex = this._zIndex;
        this._pos = this.node.position;
    }
    onTouchStart(){
        this.touchStartHandler();
    }
    // 开始
    // @syncExe
    touchStartHandler(){
        this._isDragSuccess = false;
        this._timeStamp = new Date().getTime();
        // this.node.runAction(cc.sequence(cc.scaleTo(0.12, 0.9),cc.scaleTo(0.12, 1))); 
        this.node.zIndex = 100;
        this.isMoved = false;
        // this.eventEmit(DragEvent.Drag_Start);
    }

    // 移动
    onTouchMove(event){
        if(this._isDragSuccess) return;
        let delta = event.getDelta();
        if (!this.isMoved && (Math.abs(delta.x)  > 1.5 || Math.abs(delta.y) > 1.5)) {
            this.isMoved = true;
            this.onDragMoveHandler();
        }
        this.node.x += delta.x;
        this.node.y += delta.y;
        if (new Date().getTime() - this._timeStamp > this._durTime) {
            this._timeStamp = new Date().getTime();
            this.touchMoveHandler();
        }
        // this.eventEmit( DragEvent.Drag_Move, this.node.name );
    }

    onDragMoveHandler(){
        // this.eventEmit(DragEvent.Drag_Move);
    }
    touchMoveHandler(){
        // this.sendRpc(ComponentBase.synStatusSignNo, {
        //     nodeName:this.node.name,
        //     property:{
        //         position:this.node.position,
        //         zIndex:this.node.zIndex
        //     }
        // });
    }
    // 结束
    onTouchEnd(){
        let node = this.node;
        node.zIndex = this._zIndex;
        let isDragSuccess = false;
        let targetNodeName = '';
        // 碰撞检测
        for (const targetNode of this.targetNodeList) {
            if (targetNode && targetNode.active) {
                let gapWidth = Math.abs(targetNode.x - node.x);
                let gapHeight = Math.abs(targetNode.y - node.y);
                if(gapWidth < node.width * 0.5 && gapHeight < node.height * 0.5){
                    isDragSuccess = true;
                    targetNodeName = targetNode.name;
                    break;
                }
            }
        }
        this.touchEndHandler(isDragSuccess, targetNodeName);
    }
    
    // @syncExe
    touchEndHandler(isDragSuccess:boolean, targetNodeName?:string){
        this._isDragSuccess = isDragSuccess
        if (this._isDragSuccess) {
            // this.eventEmit(DragEvent.Drag_Success,targetNodeName);
            // for (const targetNode of this.targetNodeList) {
            //     if (targetNode.name === targetNodeName) {
            //         targetNode.emit(DragEvent.Drag_Copy,this._pos ,this.node.getComponent(cc.Sprite).spriteFrame,this.targetNodeList,this.node.name);
            //         break;
            //     }
            // }
        }
        if (!this._isDragSuccess && this.targetNodeList.length > 0) {
            this.backToOriginalPos();
        }else{
            // this.node.runAction(cc.scaleTo(0.12, 1));
        }
        // this.eventEmit(DragEvent.Drag_End,this._isDragSuccess);
        // if (!this.isMoved) {
        //     this.eventEmit(DragEvent.Touch);
        // }
        this.isMoved = false;
    }

    backToOriginalPos(){
        this.node.stopAllActions();
        this.node.zIndex = this._zIndex;
        const moveAct = cc.moveTo(0.28, this._pos.x, this._pos.y).easing(cc.easeSineOut());
        // this.node.runAction(cc.spawn(moveAct, cc.scaleTo(0.28,1)));
    }
    
    getInitPos(){
        return this._pos;
    }

    eventEmit(eventName:string, param?:any){
        // 抛出事，拖拽到目标
        if (this.eventNode) {
            this.eventNode.emit(eventName, this.node.name,param);
        }else{
            this.node.emit(eventName, this.node.name,param);
        }
    }
    // update (dt) {}
}
